#!BPY

"""
Name: 'AdjustAxisScale'
Blender: 244
Group: 'Object'
"""

__author__ = "Dusan Pavlicek"
__email__ = "pavlicd@volny.cz"

import Blender
import math

from Blender import *
from Blender.Mathutils import *


BUTTON_X = 1
BUTTON_Y = 2
BUTTON_Z = 3
BUTTON_ASSIGN1 = 4
BUTTON_ASSIGN2 = 5

AXIS_X = 1
AXIS_Y = 2
AXIS_Z = 3

AXIS = AXIS_Y
SEL_VERTS1 = []
SEL_VERTS2 = []


def CalcScaleTransform(axis, p1, p2, p3, q1, q2, q3):
	a = Vector(p2.x - p1.x, p2.y - p1.y, p2.z - p1.z)
	b = Vector(p3.x - p1.x, p3.y - p1.y, p3.z - p1.z)
	c = Vector(q2.x - q1.x, q2.y - q1.y, q2.z - q1.z)
	d = Vector(q3.x - q1.x, q3.y - q1.y, q3.z - q1.z)
	u = (a.y * b.z - a.z * b.y) * (c.y * d.z - c.z * d.y)
	v = (a.z * b.x - a.x * b.z) * (c.z * d.x - c.x * d.z)
	w = (a.x * b.y - a.y * b.x) * (c.x * d.y - c.y * d.x)
	sx = 1
	sy = 1
	sz = 1
	if axis == AXIS_X:
		if v + w != 0:
			t = -u / (v + w)
		else:
			t = -1
		if t > 0:
			s = math.sqrt(t);
			sx = s
	elif axis == AXIS_Y:
		if u + w != 0:
			t = -v / (u + w)
		else:
			t = -1
		if t > 0:
			s = math.sqrt(t);
			sy = s
	elif axis == AXIS_Z:
		if u + v != 0:
			t = -w / (u + v)
		else:
			t = -1
		if t > 0:
			s = math.sqrt(t);
			sz = s
	if t > 0:
		Draw.PupMenu("Axis scale multiplied by %f." % s)
		if s == 1:
			return None
	else:
		if abs(u + v + w) < 1e-4:
			Draw.PupMenu("Planes are already perpendicular.")
		else:
			Draw.PupMenu("Sorry, I cannot make the two planes perpendicular.")
		return None
	return ScaleMatrix(sx, sy, sz)


def ScaleMatrix(sx, sy, sz):
	return Matrix(
		[sx, 0, 0, 0],
		[0, sy, 0, 0],
		[0, 0, sz, 0],
		[0, 0, 0, 1])


def GetSelectedVerts():
	origEditMode = Window.EditMode()
	if (origEditMode == 1):
		Window.EditMode(0)
	selObjs = Object.GetSelected()
	if len(selObjs) == 0:
		Window.EditMode(origEditMode)
		return []
	obj = selObjs[0]
	objMatrix = obj.getMatrix();
	objVerts = obj.getData().verts
	selVerts = [v for v in objVerts if v.sel == 1]
	selVertsCo = []
	for v in selVerts:
		selVertsCo.append(v.co * objMatrix)
	Window.EditMode(origEditMode)
	return selVertsCo


def CorrectAxisScale(axis, selectedVerts1, selectedVerts2):
	matrix = CalcScaleTransform(
			axis,
			selectedVerts1[0], 
			selectedVerts1[1], 
			selectedVerts1[2],
			selectedVerts2[0], 
			selectedVerts2[1], 
			selectedVerts2[2])
	
	if matrix != None:
		for obj in Object.Get():
			if IsObjectVisible(obj):
				obj.setMatrix(obj.getMatrix() * matrix)
	Redraw()
	return


def IsObjectVisible(object):
	visibleLayers = Window.ViewLayers()
	for layer in object.layers:
		if layer in visibleLayers:
			return True
	return False


def DrawGUI():
	YPOS = 210
	BGL.glClear(BGL.GL_COLOR_BUFFER_BIT)

	BGL.glColor3f(0, 0, 0)
	BGL.glRasterPos2d(10, YPOS)
	Draw.Text("This script adjusts the scale of a selected axis")
	BGL.glRasterPos2d(10, YPOS - 15)
	Draw.Text("so that the two given planes become perpendicular.")

	BGL.glColor3f(0, 0, 0)
	BGL.glRasterPos2d(10, YPOS - 50)
	Draw.Text("1) Select the axis whose scale you want to adjust.")
	if AXIS == AXIS_X:
		toggle = 1
	else:
		toggle = 0
	Draw.Toggle("X", BUTTON_X, 25, YPOS - 80, 30, 20, toggle)
	if AXIS == AXIS_Y:
		toggle = 1
	else:
		toggle = 0
	Draw.Toggle("Y", BUTTON_Y, 60, YPOS - 80, 30, 20, toggle)
	if AXIS == AXIS_Z:
		toggle = 1
	else:
		toggle = 0
	Draw.Toggle("Z", BUTTON_Z, 95, YPOS - 80, 30, 20, toggle)


	BGL.glColor3f(0, 0, 0)
	BGL.glRasterPos2d(10, YPOS - 100)
	Draw.Text("2) Select the")
	BGL.glColor3f(1, 1, 0)
	BGL.glRasterPos2d(85, YPOS - 100)
	Draw.Text("first set of 3 vertices")
	BGL.glColor3f(0, 0, 0)
	BGL.glRasterPos2d(201, YPOS - 100)
	Draw.Text("and press the button.")
	Draw.Button("Assign plane 1", BUTTON_ASSIGN1, 25, YPOS - 130, 100, 20)

	BGL.glColor3f(0, 0, 0)
	BGL.glRasterPos2d(10, YPOS - 150)
	Draw.Text("3) Select the")
	BGL.glColor3f(1, 1, 0)
	BGL.glRasterPos2d(85, YPOS - 150)
	Draw.Text("second set of 3 vertices")
	BGL.glColor3f(0, 0, 0)
	BGL.glRasterPos2d(223, YPOS - 150)
	Draw.Text("and press the button.")
	Draw.Button("Assign plane 2", BUTTON_ASSIGN2, 25, YPOS - 180, 100, 20)

	return


def HandleEvent(event, value):
	if event == Draw.ESCKEY:
		Draw.Exit()
	return


def HandleButton(button):
	global AXIS
	global SEL_VERTS1
	global SEL_VERTS2
	if button == BUTTON_X:
		AXIS = AXIS_X
		Draw.Redraw()
	elif button == BUTTON_Y:
		AXIS = AXIS_Y
		Draw.Redraw()
	elif button == BUTTON_Z:
		AXIS = AXIS_Z
		Draw.Redraw()
		
	if button == BUTTON_ASSIGN1:
		SEL_VERTS1 = GetSelectedVerts()
		if len(SEL_VERTS1) != 3:
			Draw.PupMenu("Select exactly 3 vertices please.")
		elif len(SEL_VERTS2) == 3:
			CorrectAxisScale(AXIS, SEL_VERTS1, SEL_VERTS2)
			SEL_VERTS1 = []
			SEL_VERTS2 = []
		return
	if button == BUTTON_ASSIGN2:
		SEL_VERTS2 = GetSelectedVerts()
		if len(SEL_VERTS2) != 3:
			Draw.PupMenu("Select exactly 3 vertices please.")
		elif len(SEL_VERTS1) == 3:
			CorrectAxisScale(AXIS, SEL_VERTS1, SEL_VERTS2)
			SEL_VERTS1 = []
			SEL_VERTS2 = []
		return
	return


def Main():
	Draw.Register(DrawGUI, HandleEvent, HandleButton)
	return
	

Main()
